import string

from ..functions import ComponentBase


class WebContentManager(ComponentBase, WebKit2.UserContentManager):
	def __init__(self, tab):
		super().__init__()

		self.app = tab.window.app
		self.scripts = DotDict()


	def __getitem__(self, key):
		self.scripts[key]


	def __setitem__(self, key, value):
		self.scripts[key] = value


	def __delitem__(self, key):
		del self.scripts[key]


	def _validate_name(self, name):
		allowed_chars = Set('-_.' + string.ascii_letters + string.digits)
		assert Set(name).issubset(allowed_chars)


	def add_script(self, path, allow=None, deny=None):
		pass


	def load_script(self, name):
		self._validate_name(name)
		with self.app.path.userscript.join(name, 'script.js').open() as fd:
			script_data = fd.read()

		self[name] = WebKit2.UserScript.new(
			script_data,
			WebKit2.UserContentInjectedFrames.ALL_FRAMES,
			WebKit2.UserContentInjectionTime.START,
			allow,
			deny
		)

		self.add_script(self[path])


	def create_script(self, name, fullname=None, inject_time='start', allow=[], deny=[]):
		self._validate_name(name)
		path = self.app.path.userscript.join(name)

		config = DotDict(
			fullname = fullname or name.replace('_', '-'),
			name = name,
			inject_time = inject_time,
			allow = allow,
			deny = deny
		)

		config.save_json(path.join('manifest.json'))
		path.join('script.js').touch()

