from .. import protocol, var
from ..extensions import WebExtensions
from ..functions import connect
from ..widgets import Download, FileChooser


class WebContext(WebKit2.WebContext):
	def __init__(self, app):
		self.app = app

		## Setup storage
		self.storage = WebKit2.WebsiteDataManager(
			base_data_directory = app.path.storage,
			base_cache_directory = app.path.cache
		)

		super().__init__(website_data_manager = self.storage)

		self.extensions = WebExtensions(self)

		self.set_property('use-system-appearance-for-scrollbars', True)
		#self.set_web_extensions_initialization_user_data(GLib.Variant.new_string('heck'))
		self.set_spell_checking_enabled(True)
		self.set_favicon_database_directory(app.path.favicon)
		self.set_web_extensions_directory(app.path.script.join('bin'))
		self.set_cache_model(WebKit2.CacheModel.WEB_BROWSER)
		self.set_use_system_appearance_for_scrollbars(True)

		## Setup Cookie storage
		self.cookiemanager = self.get_cookie_manager()
		self.cookiemanager.set_persistent_storage(app.path.cookies, WebKit2.CookiePersistentStorage.SQLITE)
		self.cookiemanager.set_accept_policy(WebKit2.CookieAcceptPolicy.NO_THIRD_PARTY)

		## Register local uri schemes
		self.security_manager = self.get_security_manager()
		self.security_manager.register_uri_scheme_as_local(var.local) #Local webui
		self.security_manager.register_uri_scheme_as_local('bsweb://')
		self.security_manager.register_uri_scheme_as_local('local://') #Renamed "file://" handler
		self.security_manager.register_uri_scheme_as_secure('oauth://')
		self.security_manager.register_uri_scheme_as_no_access(var.local)
		self.security_manager.register_uri_scheme_as_no_access('bsweb://')
		self.security_manager.register_uri_scheme_as_no_access('oauth://')

		## Setup custom protocols
		self.register_uri_scheme(var.local_proto, protocol.Local)
		self.register_uri_scheme('bsweb', protocol.LocalWeb)
		self.register_uri_scheme('sftp', protocol.Sftp)
		self.register_uri_scheme('source', protocol.Source)
		self.register_uri_scheme(protocol.Oauth.protocol, protocol.Oauth)

		## Webkit won't let me set these normally
		self.register_uri_scheme('filetp', protocol.Ftp)
		self.register_uri_scheme('local', protocol.File)

		## Connect context signals
		self.connect('download-started', self.handle_new_download)
		self.connect('user-message-received', self.extensions.handle_command)


	@property
	def favicons(self):
		return self.get_favicon_database()


	@property
	def window(self):
		return self.app.window


	def get_favicon(self, url, callback, *args, **kwargs):
		self.favicons.get_favicon(url, None, self.handle_get_favicon, (callback, args, kwargs))


	def handle_get_favicon(self, _, result, data):
		try:
			favicon = self.favicons.get_favicon_finish(result)

		except GLib.Error as e:
			if e.domain != 'WebKitFaviconDatabaseError':
				raise e

			favicon = None

		data[0](favicon, *data[1], **data[2])


	def handle_new_download(self, context, download):
		Download(download)
