from urllib.parse import unquote

from . import ProtocolRequest, handle_remote_file, list_directory

from ..functions import human_bytes


def File(webkit_request):
	request = ProtocolRequest(None, webkit_request)
	path = Path(unquote(request.path))

	if not path.exists():
		msg = f'The file ({path}) does not exist'

		return request.error(f'File not found: {request.url}')

	elif path.isdir():
		filelist = []

		for item in path.listdir(recursive=False):
			try:
				filelist.append(DotDict(
					filename = item.name,
					size = human_bytes(item.size),
					mimetype = item.mimetype,
					type = 'file' if item.isfile() else 'dir',
					user = item.user,
					group = item.group
				))

			except FileNotFoundError:
				continue

		return list_directory(request, filelist)

	if path.suffix in ['htm', 'html', 'xhtml']:
		with path.open() as fd:
			return request.response(fd.read())

	elif path.suffix in ['mhtml', 'mht']:
		return request.error('MHTML files not supported yet')

	elif not request.mimetype:
		return request.error('Cannot determine file type')

	elif request.mimetype.startswith('text') or any(map(request.url.endswith, ['js', 'css'])):
		with path.open() as fd:
			return handle_remote_file(request, fd.read())

	return request.error(f'File type not supported: {request.mimetype}')
