import mimetypes

from .. import var
from ..functions import get_app


def finish_request(request, data, ctype='text/html'):
	if isinstance(data, str):
		data = data.encode('UTF-8')

	bytestream = Gio.MemoryInputStream.new_from_bytes(GLib.Bytes(data))
	request.finish(bytestream, len(data), ctype)


def finish_request_error(request, message, title='Error'):
	return finish_request(
		request,
		get_app().template.render(
			'page/error.haml',
			dict(
				title = title,
				error_page_message = message
			)
		)
	)


def preferences_return(window, webview):
	window['library-container'].show()
	tabid = webview.get_parent().tabid
	window.close_tab(tabid)
	return True


def redirect(webview, msg, url=None, level='error'):
	window = webview.get_toplevel()
	window.notification(msg, level)
	webview.load_uri(url or var.local)
	return True


def error(webview, msg, url=None):
	window = webview.get_toplevel()
	window.notification(msg, 'error')
	webview.load_uri(url or var.local)
	return True


def list_directory(request, files, show_hidden=False):
	filelist = []
	dirlist = []

	for item in sorted(files, key=lambda d: d['filename']):
		if not show_hidden and item['filename'].startswith('.'):
			continue

		if item.type == 'dir':
			dirlist.append(item)

		elif item.type == 'file':
			filelist.append(item)

		else:
			logging.warning('Unknown item type:', item.type, item)

	return finish_request(
		request,
		get_app().template.render(
			'page/files.haml',
			dict(
				dirlist = dirlist,
				filelist = filelist,
				url = Url(request.get_uri()),
				textlines = None
			)
		)
	)


def handle_remote_file(request, url, data):
	mimetype = mimetypes.guess_type(url, strict=False)[0]

	if url.path.suffix in ['htm', 'html', 'xhtml']:
		html = data

	#elif url.path.suffix in ['py', 'ini', 'env', 'txt', 'json', 'sgrd', 'xspf', 'go', 'js', 'css', 'scss', 'log']:
	elif mimetype.startswith('text') or url.path.suffix in ['json']:
		if mimetype == 'application/json':
			output = DotDict(data).to_json(4).split('\n')

		else:
			if type(data) == bytes:
				data.decode('utf-8')

			output = data.split('\n')

		html = get_app().template.render('page/files.haml', {
			'textlines': output,
			'url': url
		})

	else:
		return finish_request_error(request, f'Unhandled mimetype: {mimetype}')

	return finish_request(request, html)
