import mimetypes, pysftp

from izzylib import get_current_user_info
from paramiko.ssh_exception import AuthenticationException
from urllib.parse import unquote

from . import ProtocolRequest, handle_remote_file, list_directory

from .. import var
from ..functions import human_bytes


def Sftp(webkit_request):
	request = ProtocolRequest(webkit_request)
	path = Path(unquote(request.path) or '/')
	keypath = Path('~/.ssh/id_rsa')
	key = keypath if keypath.exists else None

	options = {
		'username': request.url.username or get_current_user_info('name'),
		'port': request.url.port
	}

	try:
		with pysftp.Connection(request.url.host, **{k:v for k,v in options.items() if v}, private_key=key) as sftp:
			if not sftp.exists(path):
				return request.error(path, 'File or path not found')

			if sftp.isfile(path):
				with sftp.open(path) as fd:
					return handle_remote_file(request, fd.read())

			else:
				data = sftp.listdir_attr(path)
				filelist = []

				for line in data:
					filelist.append(DotDict(
						filename = line.filename,
						size = human_bytes(line.st_size),
						mimetype = mimetypes.guess_type(line.filename, strict=False),
						type = 'file' if sftp.isfile(unquote(path.join(line.filename))) else 'dir',
						user = line.st_uid,
						group = line.st_gid
					))

				return list_directory(request, filelist)

	except AuthenticationException:
		return request.error('Username or password invalid')
